#ifndef _ATCOM_SMS_H
#define _ATCOM_SMS_H

//#include "asterisk/channel.h"
//#include "asterisk/xml.h"

#if defined(__cplusplus) || defined(c_plusplus)
extern "C" {
#endif
/*jwj add 1222*/
struct ast_xml_node;
struct ast_xml_doc;

/*! \brief Initialize the XML library implementation.
 *         This function is used to setup everything needed
 *         to start working with the xml implementation.
 *  \retval 0 On success.
 *  \retval 1 On error.
 */
int ast_xml_init(void);

/*! \brief Cleanup library allocated global data.
 *  \retval 0 On success.
 *  \retval 1 On error.
 */
int ast_xml_finish(void);

/*! \brief Open an XML document.
 *  \param filename Document path.
 *  \retval NULL on error.
 *  \retval The ast_xml_doc reference to the open document.
 */
struct ast_xml_doc *ast_xml_open(char *filename);

/*! \brief Close an already open document and free the used
 *        structure.
 *  \retval doc The document reference.
 */
void ast_xml_close(struct ast_xml_doc *doc);

/*! \brief Get the document root node.
 *  \param doc Document reference
 *  \retval NULL on error
 *  \retval The root node on success.
 */
struct ast_xml_node *ast_xml_get_root(struct ast_xml_doc *doc);

/*! \brief Free node
 *  \param node Node to be released.
 */
void ast_xml_free_node(struct ast_xml_node *node);

/*! \brief Free an attribute returned by ast_xml_get_attribute()
 *  \param data pointer to be freed.
 */
void ast_xml_free_attr(const char *attribute);

/*! \brief Free a content element that was returned by ast_xml_get_text()
 *  \param text text to be freed.
 */
void ast_xml_free_text(const char *text);

/*! \brief Get a node attribute by name
 *  \param node Node where to search the attribute.
 *  \param attrname Attribute name.
 *  \retval NULL on error
 *  \retval The attribute value on success.
 */
const char *ast_xml_get_attribute(struct ast_xml_node *node, const char *attrname);

/*! \brief Find a node element by name.
 *  \param node This is the node starting point.
 *  \param name Node name to find.
 *  \param attrname attribute name to match (if NULL it won't be matched).
 *  \param attrvalue attribute value to match (if NULL it won't be matched).
 *  \retval NULL if not found
 *  \retval The node on success.
 */
struct ast_xml_node *ast_xml_find_element(struct ast_xml_node *root_node, const char *name, const char *attrname, const char *attrvalue);

/*! \brief Get an element content string.
 *  \param node Node from where to get the string.
 *  \retval NULL on error.
 *  \retval The text content of node.
 */
const char *ast_xml_get_text(struct ast_xml_node *node);

/*! \brief Get the name of a node. */
const char *ast_xml_node_get_name(struct ast_xml_node *node);

/*! \brief Get the node's children. */
struct ast_xml_node *ast_xml_node_get_children(struct ast_xml_node *node);

/*! \brief Get the next node in the same level. */
struct ast_xml_node *ast_xml_node_get_next(struct ast_xml_node *node);

/*! \brief Get the previous node in the same leve. */
struct ast_xml_node *ast_xml_node_get_prev(struct ast_xml_node *node);

/*! \brief Get the parent of a specified node. */
struct ast_xml_node *ast_xml_node_get_parent(struct ast_xml_node *node);

/* Features using ast_xml_ */
#ifdef HAVE_LIBXML2
#define AST_XML_DOCS
#endif


#define MAX_SMS_QUEUE_NUM 100
#define MAX_SMS_NUM_LEN 18
#define MAX_SMS_QUEUE_NUM_LEN (MAX_SMS_QUEUE_NUM*MAX_SMS_NUM_LEN)
#define MAX_SMS_CONTENT_LEN 1024
#define MAX_SMS_CONTENT_NUM 600
#define MAX_SMS_CONTENT_CHAR_NUM 1200
#define MAX_SMS_PDU_HEX_LEN 2410
#define MAX_SMS_QUEUE_MSG_LEN  2900
#define MAX_SMS_NORMAL_MSG_LEN 1100 

#define SMS_MAX_PDU_LEN 140
#define SMS_MAX_PDU_7BIT_LEN 160
#define SMS_MAX_TEXT_LEN 310

#define SMS_MAX_RECV_PDU_MSG_LEN 450
//#define SMS_MAX_NUM_LEN   32
#define SMS_MAX_REAL_NEWSMS_LEN 348//252


struct gsm_sms{
        unsigned int  total_len;
        int  num_len;
        unsigned int  long_flag;
        unsigned int  sms_index;
        unsigned char  num[MAX_SMS_NUM_LEN];
        unsigned char  sms_content[320];
        unsigned char  chan_id;
        unsigned char  sms_mode; //0:text 1:pdu
        unsigned char queue_sms;//0:no 1:yes
       }sms;

struct gsm_queue{
        unsigned char  queue_send;//0:single 1:queue
        unsigned char qnum[MAX_SMS_QUEUE_NUM][MAX_SMS_NUM_LEN];
};

struct pdu_package{
     int send_len;
     int calledlen;
     char conv_called[MAX_SMS_NUM_LEN];
     int longflag;
     int totalnum; 
     int smsnum;
     int longsms_id;
};

struct sms_pdu_oa{
    unsigned char len;
    unsigned char type;
    char num[MAX_SMS_NUM_LEN];
};

struct gsm_pdu_msg{
      unsigned char smsc_num_len;
      unsigned char smsc_num_type;
      char smsc_num[MAX_SMS_NUM_LEN];
      unsigned char first_byte;
      struct sms_pdu_oa oa;
      unsigned char pro_flag;
      unsigned char ud_coding; 
      char timestamp[16];
      unsigned char ud_len;
      char ud[SMS_MAX_RECV_PDU_MSG_LEN];
      int index;
      int lang;
};


struct DAHDI_GSM_RECVSMS{
        unsigned char content[SMS_MAX_REAL_NEWSMS_LEN];
        int len;//unsigned char len;
        unsigned char sendnum[MAX_SMS_NUM_LEN];
        unsigned char sendnum_len;
        unsigned char total_msg;
};

ast_mutex_t g_logfile_lock;
/***********************function*******************************/
 int sms_write_recvlog(int channum, char *sender, char *timer, char *content,char* longsms_flag);
 int sms_write_sendlog(char *sessionid,int channum,char *recver,int res);
 long utf8_to_ucs2(unsigned char **pp);
 int is_packsms8(unsigned char *o,int udl, unsigned short *ud);
 int sms_check_senddata(unsigned char *data,unsigned short *ud);
 int sms_pdu_convert_callednum(char *src_called, char *dest_called, int *called_len);

 int pdu_ucs2_to_char(unsigned char *out,unsigned short *ud,int len);
 int sms_pdu_package(char *data,unsigned char *udata,struct pdu_package *pk,int pdu_mode);
 int sms_pdu_7bit_package(char *data,unsigned char *udata,int send_len, int  calledlen, char *conv_called);
 int sms_convert_content_to_2hex(unsigned char *msg,int msg_len);
 void gsm_7bitto8Bit(unsigned char *in, unsigned char *out, int len);
 int   gsmDecode7bit(const   unsigned   char*   pSrc,   char*   pDst,   int   nSrcLength);
 int   gsmEncode7bit(const   char*   pSrc,   unsigned   char*   pDst,   int   nSrcLength);
 int  sms_pdu_decode(const unsigned char* pSrc,struct gsm_pdu_msg *pdu_msg);
 int sms_pud_ucs2_to_utf8(const char *pSrc,unsigned char *pDst);
 int dahdi_decode_pdu(void);
 int sms_decode_language(char *in, size_t inlen, char *out, size_t outlen, int id);
 int map_language_to_id(char *lang,int *id);

 unsigned long gsm_hex_to_int(char *src_hex, int len);
 void gsm_reverse_num(char *src, char *dest, int type);
 char *pdu_get_first_byte(char *src, char *first_byte);
 char *pdu_get_oa_len(char *src, char *len);
 char *pdu_get_oa_type(char *src, char *type);
 char *pdu_get_oa_num(char *src, int len, char *num);
 char *pdu_get_pro_flag(char *src, char *pro_flag);
 char *pdu_get_ud_coding(char *src, char *code);
 char *pdu_get_timestamp(char *src, char *tstamp);
 char *pdu_get_ud_len(char *src, unsigned char *len);
 void gsm_reverse_string(char *conv_dest, char *src);
 char *pdu_get_smsc_num(char *src, int len, char *dest);
 char *pdu_get_smsc_type(char *src, char *type);
 void gsm_ucs2_to_byte(unsigned char *dest,char *src,int len);

 struct ast_xml_doc *ast_xml_open_utf8(char *filename);
/*channel interface*/
//static int dahdi_send_text_sms(struct ast_channel *c,int fd,struct gsm_sms *sms);
//static int dahdi_send_pdu_sms(struct ast_channel *c,int fd,struct gsm_sms *sms);

//static int sms_do_pdu_mode(struct ast_channel *c, int fd,struct gsm_sms *sms,char *save_data,char *conv_called,int calledlen);

//static int sms_do_text_mode(struct ast_channel *c, int fd,struct gsm_sms *sms,unsigned char *udata);

//static int sms_queue_send(struct ast_channel *c,int fd,struct gsm_sms *sms,unsigned char *save_data,struct gsm_queue *qu);




#if defined(__cplusplus) || defined(c_plusplus)
}
#endif


#endif


